//
//  LKParser.m
//  LicenseKit
//
//  Created by Tanaka Katsuma on 2013/09/28.
//  Copyright (c) 2013年 Tanaka Katsuma. All rights reserved.
//

#import "LKParser.h"

#import "LKEntityText.h"
#import "LKEntityEmptyLine.h"
#import "LKEntityList.h"

@implementation LKParser

+ (NSArray *)entitiesInString:(NSString *)string
{
    NSMutableArray *entities = [NSMutableArray array];
    
    __block NSUInteger lineOffset = 0;
    
    [string enumerateLinesUsingBlock:^(NSString *line, BOOL *stop) {
        // Empty Line
        NSRegularExpression *emptyLineRegularExpression = [NSRegularExpression regularExpressionWithPattern:@"^\\s*$"
                                                                                                    options:0
                                                                                                      error:NULL];
        NSArray *emptyLineMatches = [emptyLineRegularExpression matchesInString:line options:0 range:NSMakeRange(0, line.length)];
        
        if (emptyLineMatches.count > 0) {
            // Add entity
            NSTextCheckingResult *textCheckingResult = [emptyLineMatches lastObject];
            NSRange entityRange = [textCheckingResult rangeAtIndex:0];
            
            LKEntityEmptyLine *entity = [LKEntityEmptyLine entityWithRange:NSMakeRange(lineOffset + entityRange.location, entityRange.length)];
            [entities addObject:entity];
            
            // Move offset
            NSRange lineRange = [string lineRangeForRange:NSMakeRange(lineOffset, 0)];
            lineOffset += lineRange.length;
            
            return;
        }
        
        // Ordered List
        NSRegularExpression *orderedListRegularExpression = [NSRegularExpression regularExpressionWithPattern:@"^\\s*(\\d\\.\\s*(.*))"
                                                                                                      options:0
                                                                                                        error:NULL];
        NSArray *orderedListMathces = [orderedListRegularExpression matchesInString:line options:0 range:NSMakeRange(0, line.length)];
        
        if (orderedListMathces.count > 0) {
            // Add entity
            NSTextCheckingResult *textCheckingResult = [orderedListMathces lastObject];
            NSRange entityRange = [textCheckingResult rangeAtIndex:1];
            NSRange stringRange = [textCheckingResult rangeAtIndex:2];
            
            LKEntityList *entity = [LKEntityList entityWithRange:NSMakeRange(lineOffset + entityRange.location, entityRange.length)
                                                               stringRange:NSMakeRange(lineOffset + stringRange.location, stringRange.length)
                                                                   ordered:YES];
            [entities addObject:entity];
            
            // Move offset
            NSRange lineRange = [string lineRangeForRange:NSMakeRange(lineOffset, 0)];
            lineOffset += lineRange.length;
            
            return;
        }
        
        // Unordered List
        NSRegularExpression *unorderedListRegularExpression = [NSRegularExpression regularExpressionWithPattern:@"^\\s*(\\*\\s*(.*))"
                                                                                                        options:0
                                                                                                          error:NULL];
        NSArray *unorderedListMathces = [unorderedListRegularExpression matchesInString:line options:0 range:NSMakeRange(0, line.length)];
        
        if (unorderedListMathces.count > 0) {
            // Add entity
            NSTextCheckingResult *textCheckingResult = [unorderedListMathces lastObject];
            NSRange entityRange = [textCheckingResult rangeAtIndex:1];
            NSRange stringRange = [textCheckingResult rangeAtIndex:2];
            
            LKEntityList *entity = [LKEntityList entityWithRange:NSMakeRange(lineOffset + entityRange.location, entityRange.length)
                                                               stringRange:NSMakeRange(lineOffset + stringRange.location, stringRange.length)
                                                                   ordered:NO];
            [entities addObject:entity];
            
            // Move offset
            NSRange lineRange = [string lineRangeForRange:NSMakeRange(lineOffset, 0)];
            lineOffset += lineRange.length;
            
            return;
        }
        
        // Plain Text
        if ([[entities lastObject] isMemberOfClass:[LKEntityList class]]) {
            LKEntityList *previousEntity = [entities lastObject];
            [entities removeLastObject];
            
            NSRange entityRange = NSMakeRange(previousEntity.entityRange.location,
                                              lineOffset + line.length - previousEntity.entityRange.location);
            NSRange stringRange = NSMakeRange(previousEntity.stringRange.location,
                                              lineOffset + line.length - previousEntity.stringRange.location);
            
            LKEntityList *entity = [LKEntityList entityWithRange:entityRange
                                                               stringRange:stringRange
                                                                   ordered:previousEntity.ordered];
            [entities addObject:entity];
        } else {
            NSRange entityRange = NSMakeRange(lineOffset, line.length);
            
            LKEntityText *previousEntity = [entities lastObject];
            
            if (previousEntity && [previousEntity isMemberOfClass:[LKEntityText class]]) {
                [entities removeLastObject];
                
                entityRange = NSMakeRange(previousEntity.entityRange.location,
                                          lineOffset + line.length - previousEntity.entityRange.location);
            }
            
            LKEntityText *entity = [LKEntityText entityWithRange:entityRange];
            [entities addObject:entity];
        }
        
        // Move offset
        NSRange lineRange = [string lineRangeForRange:NSMakeRange(lineOffset, 0)];
        lineOffset += lineRange.length;
    }];
    
    return [entities copy];
}

+ (NSString *)HTMLStringFromTermsString:(NSString *)string
{
    NSMutableString *HTMLString = [NSMutableString string];
    
    NSArray *entities = [self entitiesInString:string];
    BOOL inList = NO;
    BOOL ordered = NO;
    
    for (LKEntityText *entity in entities) {
        if ([entity isMemberOfClass:[LKEntityList class]]) {
            if (!inList) {
                ordered = [(LKEntityList *)entity ordered];
                [HTMLString appendFormat:@"<%@>\n", ordered ? @"ol" : @"ul"];
                
                inList = YES;
            }
            
            [HTMLString appendFormat:@"<li>%@</li>\n", [string substringWithRange:[(LKEntityList *)entity stringRange]]];
        }
        else {
            if (inList) {
                [HTMLString appendFormat:@"</%@>\n", ordered ? @"ol" : @"ul"];
                
                inList = NO;
            }
            
            if ([entity isMemberOfClass:[LKEntityText class]]) {
                [HTMLString appendFormat:@"<p>%@</p>\n", [string substringWithRange:entity.entityRange]];
            }
            else if ([entity isMemberOfClass:[LKEntityEmptyLine class]]) {
                [HTMLString appendString:@"\n"];
            }
        }
    }
    
    return HTMLString;
}

@end
